import { Blip } from '../Blip';
import { ForceType } from '../enums';
import { WeaponHash } from '../hashes';
import { Model } from '../Model';
import { Vector3 } from '../utils';
import { EntityBoneCollection, Ped, Prop, Vehicle } from './';
import { EntityBone } from './EntityBone';
export declare class Entity {
    static fromHandle(handle: number): Ped | Vehicle | Prop;
    static fromNetworkId(networkId: number): Ped | Vehicle | Prop;
    protected handle: number;
    protected bones: EntityBoneCollection;
    constructor(handle: number);
    get Handle(): number;
    get NetworkId(): number;
    get Health(): number;
    set Health(amount: number);
    get MaxHealth(): number;
    set MaxHealth(amount: number);
    isDead(): boolean;
    isAlive(): boolean;
    get Position(): Vector3;
    set Position(position: Vector3);
    set PositionNoOffset(position: Vector3);
    get Heading(): number;
    set Heading(heading: number);
    get IsCollisionEnabled(): boolean;
    set IsCollisionEnabled(value: boolean);
    set IsRecordingCollisions(value: boolean);
    get Bones(): EntityBoneCollection;
    get AttachedBlip(): Blip;
    attachBlip(): Blip;
    setNoCollision(entity: Entity, toggle: boolean): void;
    hasBeenDamagedBy(entity: Entity): boolean;
    hasBeenDamagedByWeapon(weapon: WeaponHash): boolean;
    hasBeenDamagedByAnyWeapon(): boolean;
    hasBeenDamagedByAnyMeleeWeapon(): boolean;
    clearLastWeaponDamage(): void;
    isInArea(minBounds: Vector3, maxBounds: Vector3): boolean;
    isInAngledArea(origin: Vector3, edge: Vector3, angle: number): boolean;
    isInRangeOf(position: Vector3, range: number): boolean;
    isNearEntity(entity: Entity, bounds: Vector3): boolean;
    isTouching(entity: Entity): boolean;
    isTouchingModel(model: Model): boolean;
    getOffsetPosition(offset: Vector3): Vector3;
    getPositionOffset(worldCoords: Vector3): Vector3;
    attachTo(entity: Entity, position: Vector3, rotation: Vector3): void;
    attachToBone(entityBone: EntityBone, position: Vector3, rotation: Vector3): void;
    detach(): void;
    isAttached(): boolean;
    isAttachedTo(entity: Entity): boolean;
    getEntityAttachedTo(): Entity;
    applyForce(direction: Vector3, rotation: Vector3, forceType?: ForceType): void;
    applyForceRelative(direction: Vector3, rotation: Vector3, forceType?: ForceType): void;
    removeAllParticleEffects(): void;
    exists(): boolean;
    delete(): void;
    markAsNoLongerNeeded(): void;
}
